@@ -20,6 +20,11 @@ enum {
 };
 static guint signals[LAST_SIGNAL] = { 0, };
 
+enum {
+    PROP_0,
+    PROP_PAGE
+};
+
 typedef enum
 {
     CROP_NONE = 0,
@@ -74,9 +79,9 @@ G_DEFINE_TYPE (PageView, page_view, G_TYPE_OBJECT);
 
 
 PageView *
-page_view_new (void)
+page_view_new (Page *page)
 {
-    return g_object_new (PAGE_VIEW_TYPE, NULL);
+    return g_object_new (PAGE_VIEW_TYPE, "page", page, NULL);
 }
 
 
@@ -426,6 +431,7 @@ update_preview (GdkPixbuf *image,
     g_return_if_fail (R < output_width);
     g_return_if_fail (T >= 0);
     g_return_if_fail (B < output_height);
+    g_return_if_fail (*output_image != NULL);
 
     output = gdk_pixbuf_get_pixels (*output_image);
     output_rowstride = gdk_pixbuf_get_rowstride (*output_image);
@@ -452,6 +458,20 @@ update_preview (GdkPixbuf *image,
 }
 
 
+static gint
+get_preview_width (PageView *view)
+{
+    return view->priv->width - view->priv->border_width * 2;
+}
+
+
+static gint
+get_preview_height (PageView *view)
+{
+    return view->priv->height - view->priv->border_width * 2;
+}
+
+
 static void
 update_page_view (PageView *view)
 {
@@ -467,8 +487,8 @@ update_page_view (PageView *view)
 
     update_preview (image,
                     &view->priv->image,
-                    view->priv->width - view->priv->border_width * 2,
-                    view->priv->height - view->priv->border_width * 2,
+                    get_preview_width (view),
+                    get_preview_height (view),
                     page_get_orientation (view->priv->page), old_scan_line, scan_line);
     g_object_unref (image);
 
@@ -480,28 +500,28 @@ update_page_view (PageView *view)
 static gint
 page_to_screen_x (PageView *view, gint x)
 {
-    return (double) x * gdk_pixbuf_get_width (view->priv->image) / page_get_width (view->priv->page) + 0.5;
+    return (double) x * get_preview_width (view) / page_get_width (view->priv->page) + 0.5;
 }
 
 
 static gint
 page_to_screen_y (PageView *view, gint y)
 {
-    return (double) y * gdk_pixbuf_get_height (view->priv->image) / page_get_height (view->priv->page) + 0.5;    
+    return (double) y * get_preview_height (view) / page_get_height (view->priv->page) + 0.5;    
 }
 
 
 static gint
 screen_to_page_x (PageView *view, gint x)
 {
-    return (double) x * page_get_width (view->priv->page) / gdk_pixbuf_get_width (view->priv->image) + 0.5;
+    return (double) x * page_get_width (view->priv->page) / get_preview_width (view) + 0.5;
 }
 
 
 static gint
 screen_to_page_y (PageView *view, gint y)
 {
-    return (double) y * page_get_height (view->priv->page) / gdk_pixbuf_get_height (view->priv->image) + 0.5;
+    return (double) y * page_get_height (view->priv->page) / get_preview_height (view) + 0.5;
 }
 
 
@@ -518,8 +538,8 @@ get_crop_location (PageView *view, gint x, gint y)
         return 0;
 
     page_get_crop (view->priv->page, &cx, &cy, &cw, &ch);
-    dx = page_to_screen_x (view, cx) + view->priv->x;
-    dy = page_to_screen_y (view, cy) + view->priv->y;
+    dx = page_to_screen_x (view, cx);
+    dy = page_to_screen_y (view, cy);
     dw = page_to_screen_x (view, cw);
     dh = page_to_screen_y (view, ch);
     ix = x - dx;
@@ -801,13 +821,13 @@ page_view_render (PageView *view, cairo_t *context)
     gint width, height;
 
     g_return_if_fail (view != NULL);
-  
-    update_animation (view);
+    g_return_if_fail (context != NULL);
 
-    /* Regenerate page pixbuf */
+    update_animation (view);
     update_page_view (view);
-    width = gdk_pixbuf_get_width (view->priv->image);
-    height = gdk_pixbuf_get_height (view->priv->image);
+
+    width = get_preview_width (view);
+    height = get_preview_height (view);
 
     cairo_set_line_width (context, 1);
     cairo_translate (context, view->priv->x, view->priv->y);
@@ -824,8 +844,18 @@ page_view_render (PageView *view, cairo_t *context)
 
     /* Draw image */
     cairo_translate (context, view->priv->border_width, view->priv->border_width);
-    gdk_cairo_set_source_pixbuf (context, view->priv->image, 0, 0);
-    cairo_paint (context);
+    if (view->priv->image) {
+        gdk_cairo_set_source_pixbuf (context, view->priv->image, 0, 0);
+        cairo_paint (context);
+    }
+    else {
+        cairo_scale (context,
+                     (double) get_preview_width (view) / page_get_width (view->priv->page),
+                     (double) get_preview_height (view) / page_get_height (view->priv->page));
+        gdk_cairo_set_source_pixbuf (context, page_get_image (view->priv->page), 0, 0);
+
+        cairo_paint (context);
+    }
 
     /* Draw throbber */
     if (page_is_scanning (view->priv->page) && !page_has_data (view->priv->page)) {
@@ -1019,7 +1049,7 @@ page_overlay_changed_cb (Page *p, PageView *view)
 }
 
 
-void
+static void
 page_view_set_page (PageView *view, Page *page)
 {
     g_return_if_fail (view != NULL);
@@ -1033,6 +1063,48 @@ page_view_set_page (PageView *view, Page *page)
 }
 
 
+static void
+page_view_set_property (GObject      *object,
+                        guint         prop_id,
+                        const GValue *value,
+                        GParamSpec   *pspec)
+{
+    PageView *self;
+
+    self = PAGE_VIEW (object);
+
+    switch (prop_id) {
+    case PROP_PAGE:
+        page_view_set_page (self, g_value_get_object (value));
+        break;
+    default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        break;
+    }
+}
+
+
+static void
+page_view_get_property (GObject    *object,
+                        guint       prop_id,
+                        GValue     *value,
+                        GParamSpec *pspec)
+{
+    PageView *self;
+
+    self = PAGE_VIEW (object);
+
+    switch (prop_id) {
+    case PROP_PAGE:
+        g_value_set_object (value, self->priv->page);
+        break;
+    default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        break;
+    }
+}
+
+
 static void
 page_view_finalize (GObject *object)
 {
@@ -1054,6 +1126,8 @@ page_view_class_init (PageViewClass *klass)
 {
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+    object_class->get_property = page_view_get_property;
+    object_class->set_property = page_view_set_property;
     object_class->finalize = page_view_finalize;
 
     signals[CHANGED] =
@@ -1074,6 +1148,14 @@ page_view_class_init (PageViewClass *klass)
                       G_TYPE_NONE, 0);
 
     g_type_class_add_private (klass, sizeof (PageViewPrivate));
+
+    g_object_class_install_property (object_class,
+                                     PROP_PAGE,
+                                     g_param_spec_object ("page",
+                                                          "page",
+                                                          "Page being rendered",
+                                                          PAGE_TYPE,
+                                                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
 
