@@ -13,6 +13,7 @@
 #define _PAGE_H_
 
 #include <glib-object.h>
+#include <gio/gio.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include "scanner.h"
 
@@ -55,6 +56,8 @@ GType page_get_type (void);
 Page *page_new (void);
 
 // FIXME: Should be part of page_new
+void page_setup (Page *page, gint width, gint height, gint dpi, Orientation orientation);
+
 void page_set_scan_area (Page *page, gint width, gint rows, gint dpi);
 
 gint page_get_dpi (Page *page);
@@ -69,6 +72,10 @@ gint page_get_scan_width (Page *page);
 
 gint page_get_scan_height (Page *page);
 
+void page_set_color_profile (Page *page, const gchar *color_profile);
+
+const gchar *page_get_color_profile (Page *page);
+
 void page_start (Page *page);
 
 gboolean page_is_scanning (Page *page);
@@ -109,10 +116,6 @@ GdkPixbuf *page_get_image (Page *page);
 
 GdkPixbuf *page_get_cropped_image (Page *page);
 
-gboolean page_save_jpeg (Page *page, const gchar *uri, GError **error);
-
-gboolean page_save_png (Page *page, const gchar *uri, GError **error);
-
-gboolean page_save_tiff (Page *page, const gchar *uri, GError **error);
+gboolean page_save (Page *page, const gchar *type, GFile *file, GError **error);
 
 #endif /* _PAGE_H_ */
